/* $Id: CalcSpectrogram.java,v 1.10 2011/01/09 08:50:52 mit Exp $ */
/* CalcSpectrogram.java v0.1.1 */

public class CalcSpectrogram {
    private boolean Lsw = true;
    private int NN, N2;
    private int[] wave;
    private int[] temp;
    private double[] Wind;
    private double[] YY;
    private double[] Re;
    private double[] Im;
    private FFT fftN;

    private static int MaxData;
    private static int Frame;
    private static double MinD, MaxD;
    private static double[] Spec;

    public CalcSpectrogram(int[] data, int nn) {
	wave = data;
	initSpectrogram(nn);
    }
    public void initSpectrogram(int nn) {
	NN = nn;
	N2 = NN/2;

	fftN = new FFT(NN);
	YY = new double[N2];

	initWind();
    }
    public void calcSpectrogram(int dev, int start, int wide, boolean sw) {
	Lsw = sw;
	int offset, shift = NN/dev;
	Frame = (int)Math.floor((wide-(NN-shift))/shift); 
	temp = new int[NN];
	MaxData = N2*Frame;
	Spec = new double[MaxData];
	MaxD = MinD = 0;

	for (int i = 0; i < Frame; i++) {
	    offset = shift*i+start;
	    System.arraycopy(wave, offset, temp, 0, NN);

	    fft(temp);

	    System.arraycopy(YY, 0, Spec, i*N2, N2);
	}
    }
    //
    public void initWind() {
	initGauss(); // initHanning();
    }
    public void wind(int[] ad) {
	gauss(ad); // hanning(ad);
    }
    public void initGauss() {
        Wind = new double[NN];
        double in, rn = 1.0/NN;

        for (int i = 0; i < NN; i++) {
            in = (2.0*i-NN)*rn;
            Wind[i] = Math.exp(-4.5*in*in);
        }
    }
    public void gauss(int[] ad) {
        for (int i = 0; i < NN; i++)
            Re[i] = Im[i] = ad[i]*Wind[i];
    }
    public void initHanning() {
	Wind = new double[NN];
	double p2 = Math.PI*2;
	double lps1 = 1.0/(NN-1);

        for (int i = 0; i < NN; i++)
            Wind[i]= 0.5-0.5*Math.cos(p2*i*lps1);
    }
    public void hanning(int[] ad) {
        for (int i = 0; i < NN; i++)
            Re[i] = Im[i] = ad[i]*Wind[i];
    }
    public void fft(int[] n) {
        Re = new double[NN];
        Im = new double[NN];

        wind(n);
        fftN.fft(Re, Im);
        for (int i = 0; i < N2; i++) {
            YY[i] = Math.sqrt(Re[i]*Re[i]+Im[i]*Im[i]);
	    if (Lsw)
                YY[i] = log_10(YY[i]);

	    MaxD = Math.max(MaxD, YY[i]);
	    MinD = Math.min(MaxD, YY[i]);
        }
    }
    public void setLsw(boolean sw) {
	Lsw = sw;
    }
    public double log_10(double value) {
        return Math.log(value)/Math.log(10);
    }
    //---
    public static double[] getData() {
        return Spec;
    }
    public static double getData(int num) {
	if (num < 0 || num >= MaxData)
	    return 0;
	return Spec[num];
    }
    public static double getMaxD() {
	return MaxD;
    }
    public static double getMinD() {
	return MinD;
    }
    public static int getFrame() {
	return Frame;
    }
}