/* $Id: CalcWavelet.java,v 1.5 2008/01/13 06:13:52 mit Exp $ */
/* CalcWavelet.java v0.1 */

public class CalcWavelet {
    private final int Odiv = 24;
    private final double Sigma = 4.0;

    private int Cnt;
    private int[] Input;
    private Gabor Gb;

    private static int ViewX = 512, ViewY;
    private static double MaxValue;
    private static double[] Wlet;

    public CalcWavelet(int[] data, int cnt) {
	Input = data;
	Cnt = cnt;
    }
    public void calcWavelet(int start, int octn, int len) {
      	double d = Sigma*(0x01 << octn);
        int offset = (int)Math.sqrt(-2.0*d*d*Math.log(0.01));
        int worksz = len+offset*2;
        int[] work = new int[worksz];
        int ao = start-offset;
        for (int i = 0; i < worksz; i++)
            work[i] = ReadData(ao+i);

        Gb = new Gabor(work, worksz);
        ViewY = octn*Odiv;
        if (ViewY > 512)
            ViewY = 512;

        int l = 0;
        MaxValue = 0;
        Wlet = new double[ViewX*ViewY];
        for (int i = 0; i < ViewY; i++) {
            Gb.SetParameters(Sigma, Math.PI,
                             Math.pow(2.0, (double)i/Odiv));

            for (int j = 0; j < ViewX; j++) {
                Wlet[l] = Gb.Wavelets((int)(offset+(double)len*j/ViewX));
                if (Wlet[l] > MaxValue)
                    MaxValue = Wlet[l];
                l++;
            }
        }
    }
    public int ReadData(int p) {
        return (p < 0 || p >= Cnt) ? 0 : Input[p];
    }
    //---
    public static double[] getData() {
	return Wlet;
    }
    public static double getData(int num) {
	return Wlet[num];
    }
    public static double getMaxValue() {
	return MaxValue;
    }
    public static int getViewX() {
	return ViewX;
    }
    public static int getViewY() {
	return ViewY;
    }
}
