/* $Id: Gabor.java,v 1.1 2007/12/14 09:17:08 mit Exp $ */
/* Gabor.java v0.1 */

public class Gabor {
    static double RATE = 0.01;

    int Offset;
    int GaborN;
    int InputN;
    int[] Input;
    double[] GaborR;
    double[] GaborI;

    public Gabor(int[] input, int len) {
	Input = input;
	InputN = len;
    }

    public void SetParameters(double sigma, double omega, double a) {
	if (sigma <= 0.0 || omega <= 0.0 || a <= 0.0)
	    return;

	double ss = 2.0*sigma*sigma;
	Offset = (int)Math.sqrt(-ss*a*a*Math.log(RATE));
	GaborN = Offset*2+1;
	GaborR = new double[GaborN];
	GaborI = new double[GaborN];

	double x, d, ox;
	double ma = Math.sqrt(a);
	double ms = Math.sqrt(Math.PI*ss);
	for (int i = 0; i < GaborN; i++) {
	    x = (double)(i-Offset)/a;
	    d = Math.exp(-x*x/ss)/ms;
	    ox = omega*x;
	    GaborR[i] = d*Math.cos(ox)/ma;
	    GaborI[i] = d*Math.sin(ox)/ma;
	}
    }

    public double Wavelets(int b) {
	double dr = 0, di = 0;
	double r0, i0, r1, i1;

	for (int i = -Offset; i <= Offset; i++) {
	    r0 = (double)ReadInput(b+i);
	    i0 = 0.0;

	    r1 = ReadGaborR(i);
	    i1 = ReadGaborI(i);

	    dr += ComplexMultiplyR(r0, i0, r1, -i1);
	    di += ComplexMultiplyI(r0, i0, r1, -i1);
	}

	//return  dr*dr+di*di;
	return Math.sqrt(dr*dr+di*di); // ϥ٥ĴѤѹ
    }

    public int ReadInput(int x) {
	return (x < 0 || x >= InputN) ? 0 : Input[x];
    }
    public double ReadGaborR(int x) {
	x += Offset;
	return (x < 0 || x >= GaborN) ? 0.0 : GaborR[x];
    }
    public double ReadGaborI(int x) {
	x += Offset;
	return (x < 0 || x >= GaborN) ? 0.0 : GaborI[x];
    }
    public double ComplexMultiplyR(double r0, double i0, double r1, double i1) {
	return r0*r1-i0*i1;
    }
    public double ComplexMultiplyI(double r0, double i0, double r1, double i1) {
	return i0*r1-r0*i1;
    }
}