/* $Id: Player.java,v 1.7 2010/12/04 00:02:26 mit Exp $ */
/* Player.java v0.1 */
import java.io.*;
import javax.sound.sampled.*;

public class Player implements Runnable {
    private boolean Runsw;
    private double[] Data;
    private int BufSize;
    private int Gain = 2;
    private double MaxValue;
    private String Error = "";

    private Thread pt;
    private SourceDataLine SDLine;
    private AudioFormat AFormat;
    private final int SampleRate = 44100;

    public Player() {}
    public Player(double[] wave) {
	init_format();
	Data = wave;
    }
    public Player(double[] wave, int gain, double maxdata) {
	init_format();
	Data = wave;
	this.setGain(gain);
	this.setMaxData(maxdata);
    }
    public boolean isRun() {
        return Runsw;
    }
    public AudioFormat getFormat() {
	return AFormat;
    }
    public void setGain(int gain) {
	Gain = gain;
    }
    public void setMaxData(double max) {
	MaxValue = max;
    }
    public void init_format() {
	BufSize = SampleRate;
        AFormat = new AudioFormat(SampleRate, 16, 1, true, false);
        /* PCM_SIGNED, 44100.0 Hz, 16 bit, mono, little-endian  */
 
	DataLine.Info info = new DataLine.Info(SourceDataLine.class,
					      AFormat, BufSize);  
        if (!AudioSystem.isLineSupported(info))
            System.out.println("Line type not supported: "+info);
        try {
            SDLine = (SourceDataLine)AudioSystem.getLine(info); 
            SDLine.open();                        // ϥ饤
        } catch (LineUnavailableException e) {
            e.printStackTrace();
            System.exit(0);
        }
    }
    public byte[] double2byte(double[] data) {
        byte[] temp = new byte[BufSize*2];
        int bn = 256;

        for (int i = 0; i < BufSize; i++) {
	    temp[i*2+1] = (byte)(data[i]/bn);
            temp[i*2] = (byte)(data[i]%bn);
        }

	return temp;
    }
    //
    public void Start() {
	Runsw = true;

	pt = new Thread(this);
	pt.start();
    }
    public void Stop() {
	SDLine = null;
	Runsw = false;

	if (pt != null)
	    pt = null;
    }
    public void run() {
	byte tmp[] = new byte[BufSize];
	double wrk[] = new double[BufSize];
	boolean sw = false;
	int cnt = 0;

	SDLine.start();
	try {
	    double val = Short.MAX_VALUE/Gain/MaxValue;

	    while (isRun()) {
		for (int i = 0; i < BufSize; i++) {
		    if (cnt >= Data.length) { // û0
			wrk[i] = 0;
			sw = true;
		    } else {
			wrk[i] = val*Data[cnt++];
		    }
		}
		tmp = double2byte(wrk);
		SDLine.write(tmp, 0, tmp.length);
		if (sw) {
		    Runsw = false;
		    break;
		}
		if (Thread.interrupted()) {
		    Stop();
		    break;
		}
	    }
	} catch (Exception e) {
	    Error = "Run error."+e;
	}

	Stop();
    }
}
