/* $Id: Waviewer.java,v 1.94 2013/09/15 12:06:28 mit Exp $ */
/* Waviewer.java v0.4.1 */
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.sound.sampled.*;
import java.text.DecimalFormat;

public class Waviewer extends JPanel
    implements ActionListener, AdjustmentListener,
	       MouseListener, MouseMotionListener, Runnable
{
    private JButton jbLoad;
    private JButton jbPwide;
    private JButton jbMwide;
    private JButton jbPgain;
    private JButton jbMgain;
    private JButton jbLog;
    private JButton jbRevc;
    private JButton jbFilter;
    private JButton jbPlay;
    private JButton[] jbView;
    private JTextField jtFile;
    private JTextField jtForm;
    private JScrollBar jscStart;
    private JComboBox jcSample;
    private JComboBox jcOctave;
    private JComboBox jcDevid;
    private JComboBox jcMem;
    private JComboBox jcBand;
    private JComboBox jcGain;
    private Thread thr;

    private PaintGraph PG;
    private WaveReader WR;
    private CalcFFT CF;
    private CalcFilter CFL;
    private CalcWavelet CW;
    private CalcSpectrogram CS;
    private MemBurg MB;
    private FWT FW;
    private Player PW;
    private DecimalFormat df1;
    private DecimalFormat df0;

    private boolean CALF;   // fwt
    private boolean CALC;   // cwt
    private boolean CALS;   // spectrogram
    private boolean CALM;   // mem
    private boolean CALL;   // filter
    private boolean RevC;   // for spectrogram
    private boolean LogSW;  // log sw.
    private boolean MouSW;  // mouse sw.
    private boolean CurSW;  // cursol sw. 
    private boolean DragSW; // for wave
    private boolean FftSW;  // FFT line-barؤ
    private int FftCnt;     //  "
    private int Grap;       // ѹ
    private int Wwide;      // Wave 
    private int Wsize;      // Wave Ĺ
    private int Wstart;     // Wave ɽ
    private int Firstx;     // wave dragg
    private int Lastx;      // "
    private int Splngf;     // ץ󥰼ȿ
    private int OctaveN;    // Waveletȿ
    private int Dev;        // spectrogram/3DFFT ե
    private int NB, NN, N2;
    private int Wby;        // Wave wideܿ
    private int Mmax;       // MemBurg
    private int BB;         // filter band
    private int FF;         // filter freq.
    private int Gain;
    private double DC;      // spectrogram ܿ
    private double Gainc;   // Wavelet ܿ
    private double WaveV = Short.MAX_VALUE;
    private double FftV = 0.0001;
    private double F3dV;
    private double LogV = 10;
    private double SqrtV = 0.00005;
    private double MemV = 0.001;
    private double FwtV = 0.001;
    private File Fname;
    private String Error;
    private int[] Wave;
    private int[] Wtmp;

    private final int WAVE = 0;
    private final int FFT_ = 1;
    private final int FFT2D = 2;
    private final int FFT3D = 3;
    private final int SPECT = 4;
    private final int FWT_ = 5;
    private final int CWT_ = 6;
    private final int MEM_ = 7;
    private final int FILT = 8;
    private final int[] Sample = {9, 10, 11, 12, 13, 14};
    private final String[] Views =
    {"Wave","FFT","2DFFT","3DFFT","Spect.","FWT","CWT","MEM"};
    private final String[] Devid = {"1/1","1/2","1/4","1/8","1/16","1/32"};
    private final String[] Burg = {"G", "C", "FPE", "AIC"};

    public Waviewer() {
      	PG = new PaintGraph();
	CF = new CalcFFT();
	CFL = new CalcFilter();
	FW = new FWT();

        Border border = new BevelBorder(BevelBorder.RAISED);
        String name = "Java Waviewer";
        border = new TitledBorder(border, name);
        setBorder(border);
        df1 = new DecimalFormat("#0.0#");
	df0 = new DecimalFormat("#0.#");

	Gain = 6; // 6~2(5~1)
	Gainc = 1;
	Dev = 1;  // 1/1
	DC = 1;
	Wby = 2;
	LogSW = false;
	NB = 12;

	buildLayout();
	set_nn();
	set_mmax();
	init_log();
    }
    public static void main(String[] args) {
        Waviewer jw = new Waviewer();

        JFrame f = new JFrame();
        f.getContentPane().add(jw);
        f.pack();
        f.setSize(520, 480);
        f.setVisible(true);
        f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    }
    public void buildLayout() {
        setLayout(new BorderLayout());

	JPanel jn = new JPanel();
        jn.setLayout(new GridLayout(2,1,0,0));

	JPanel n0 = new JPanel();
	n0.setLayout(new FlowLayout());
	jn.add(n0);

        jbPlay = new JButton("F.Play");
        jbPlay.addActionListener(this);
        n0.add(jbPlay);

        int maxgain = 5;
	Integer gain[] = new Integer[maxgain];
        for (int i = 0; i < maxgain; i++) {
	    gain[i] = new Integer(i+1);
        }
	jcGain = new JComboBox(gain);
	jcGain.setSelectedIndex(0);    // 1
	jcGain.addActionListener(this);
        n0.add(jcGain);
 
        jtFile = new JTextField(16);
        jtFile.setEditable(true);
        n0.add(jtFile);

        jbLoad = new JButton("Load");
        jbLoad.addActionListener(this);
        n0.add(jbLoad);

	Integer sam[] = new Integer[Sample.length];
	for (int i = 0; i < Sample.length; i++)
            sam[i] = new Integer(Sample[i]);
	jcSample = new JComboBox(sam);
	jcSample.setSelectedIndex(3);    // 12
	jcSample.addActionListener(this);
	n0.add(jcSample);
	//

	JPanel n1 = new JPanel();
	n1.setLayout(new FlowLayout());
	jn.add(n1);

        jbFilter = new JButton("F.Filter");
        jbFilter.addActionListener(this);
        n1.add(jbFilter);

        int maxband = 10;
	Integer band[] = new Integer[maxband+1];
        for (int i = 0; i < maxband+1; i++) {
            int p = (i == 0) ? 0 : 1;
            band[i] = new Integer((int)(Math.pow(2,i)+p));
        }
	jcBand = new JComboBox(band);
	jcBand.setSelectedIndex(maxband);
	jcBand.addActionListener(this);
        n1.add(jcBand);
        BB = (int)(Math.pow(2, maxband)/2);

	int part = 8;
        Integer prt[] = new Integer[part];
        for (int i = 0; i < part; i++)    // 3~10
            prt[i] = new Integer(i+3);
        jcOctave = new JComboBox(prt);
        jcOctave.setSelectedIndex(3);     // 6
        jcOctave.addActionListener(this);
        n1.add(jcOctave);
        OctaveN = 6;

        jcDevid = new JComboBox(Devid);
        jcDevid.setSelectedIndex(0);      // 1/1
        jcDevid.addActionListener(this);
        n1.add(jcDevid);

        jbLog = new JButton("Log ");
        jbLog.addActionListener(this);
        n1.add(jbLog);

        jbRevc = new JButton("Rev.");
        jbRevc.addActionListener(this);
        n1.add(jbRevc);

	jcMem = new JComboBox(Burg);
	jcMem.addActionListener(this);
	n1.add(jcMem);

	Border bo = new SoftBevelBorder(BevelBorder.RAISED);
        jn.setBorder(bo);

        add(jn, "North");
        //-------

        JPanel jw = new JPanel();
        jw.setLayout(new GridLayout(Views.length,1,0,0));

	jbView = new JButton[Views.length];
	for (int i = 0; i < Views.length; i++) {
	    jbView[i] = new JButton(Views[i]);
	    jbView[i].addActionListener(this);
	    jw.add(jbView[i]);
	}

	add(jw, "West");
        jw.setBorder(bo);
	//--------

        JPanel je = new JPanel();
        je.setLayout(new GridLayout(5,1,0,0));

        jbPgain = new JButton("+");
        jbPgain.addActionListener(this);
        je.add(jbPgain);

        jbMgain = new JButton("-");
        jbMgain.addActionListener(this);
        je.add(jbMgain);

        je.add(new JLabel(""));

        jbPwide = new JButton(">");
        jbPwide.addActionListener(this);
        je.add(jbPwide);

        jbMwide = new JButton("<");
        jbMwide.addActionListener(this);
        je.add(jbMwide);

        add(je, "East");
        je.setBorder(bo);
        //------

        JPanel js = new JPanel();
        js.setLayout(new GridLayout(2,1,0,0));

        jscStart = new JScrollBar(JScrollBar.HORIZONTAL, 0, 100, 0, 100);
        jscStart.addAdjustmentListener(this);
        js.add(jscStart);

        jtForm = new JTextField(35);
        jtForm.setEditable(false);
        js.add(jtForm);

        add(js, "South");
        js.setBorder(bo);
        //-------------

	add(PG, "Center");

        PG.addMouseListener(this);
        PG.addMouseMotionListener(this);

	enable_set();
    }
    //
    public void run() {
        int stime = 80;
	Error = "";

        try {
	    while (PW.isRun()) {
		PG.repaint();
                thr.sleep(stime);
            }
        } catch (InterruptedException e) {
            Error = "Run error.";
        }

	thread_stop();
	set_play_button();
	PG.repaint();
    }
    public void thread_start() {
        thr = new Thread(this);
        thr.start();
    }
    public void thread_stop() {
        if (isThread())
            thr = null;
    }
    public boolean isThread() {
	return (thr != null); 
    }
    //
    public void mousePressed(MouseEvent m) {
	switch (Grap) {
	case FFT_:
	    MouSW = !MouSW;
	    PG.reset_fft_m(m.getX()); // for wide
	    if (m.getModifiers() == InputEvent.BUTTON3_MASK)
		PG.repaint();         // ܥ reset
	    break;
	case FFT2D:
	case FFT3D:
	    if (m.getModifiers() == InputEvent.BUTTON3_MASK)
		MouSW = !MouSW;       // ܥ X:Y
	    else
		CurSW = !CurSW;       // ܥǥ
	    PG.repaint();
	    break;
	case FILT:
	    CurSW = !CurSW;
	    PG.repaint();
	    break;
	case WAVE:
	    Firstx = minmax_limit(m.getX());
	    break;
	default:
	}
    }
    public void mouseDragged(MouseEvent m) {
	switch (Grap) {
	case FFT_:
	    PG.set_m_dragged(m.getX());
	    PG.repaint();
	    break;
	case WAVE:
	    DragSW = true;
	    Lastx = minmax_limit(m.getX());
            PG.repaint();
	    break;
	default:
 	}
    }
    public void mouseReleased(MouseEvent m) {
	if (Grap == WAVE) {
            DragSW = false;
	    Lastx = minmax_limit(m.getX());
	    chech_last_farst();
	    reset_wide();
            PG.repaint();
	}
    }
    public double get_LFD(double dw) {
        return (Lastx-Firstx)/dw;
    }
    public int minmax_limit(int gx) {
	if (gx > PG.W)
            gx = PG.W-1;
        if (gx < PG.LeftS)
            gx = PG.LeftS;

        return gx;
    }
    public void chech_last_farst() {
	if (Lastx < Firstx) {
	    int tmp = Lastx;
	    Lastx = Firstx;
	    Firstx = tmp;
	}
    }
    public void reset_wide() {
	double dw = (PG.W-PG.LeftS)/(double)Wwide;
	int wid = (int)(Math.abs(get_LFD(dw)));
	if (wid > NN) {
	    Wstart += (int)((Firstx-PG.LeftS)/dw);
	    Wwide = wid;
	    jscStart.setValues(Wstart, Wwide, 0, Wsize);
	}
    }
    public void mouseMoved(MouseEvent m) {
	switch (Grap) {
	case FFT2D:
	case FFT3D:
	    if (MouSW)        // xyư
		PG.set_3d_xy(m.getX(), m.getY());
	    if (CurSW) {      // freqѹ
		PG.cursolSet(m.getX());
		CALL = false;
	    }
	    PG.repaint();
	    break;
	case FILT:
	    if (CurSW)
		PG.cursolSet(m.getX());
	    CALL = false;     // Ǥưϲ
	    PG.repaint();
	    break;
	default:
 	}
    }
    public void mouseEntered(MouseEvent m) {}
    public void mouseExited(MouseEvent m) {}
    public void mouseClicked(MouseEvent m) {}
    //
    public void adjustmentValueChanged(AdjustmentEvent e) {
        if (e.getSource() == jscStart) {
	    setScroll();
	    PG.repaint();
	}
    }
    public void actionPerformed(ActionEvent e){
        if (e.getSource() == jbLoad)
            file_set();

        if (e.getSource() == jbPwide)
            set_wide(true);
        if (e.getSource() == jbMwide)
            set_wide(false);

	if (e.getSource() == jbPgain)
	    set_gain(true);
	if (e.getSource() == jbMgain)
	    set_gain(false);

	for (int i = 0; i < jbView.length; i++) {
	    if (e.getSource() == jbView[i]) {
		Grap = i;
		reset_scroll();
		set_fft_button();
		break;
	    }
	}
	if (e.getSource() == jbFilter)
	    set_filter();

	if (e.getSource() == jbPlay)
	    play_wave(); // write_data(); // for Debug

	if (e.getSource() == jcGain)
	    set_gain();

	if (e.getSource() == jcSample)
	    select_sample();

	if (e.getSource() == jcOctave)
	    select_octave();

        if (e.getSource() == jcDevid)
            select_devid();

	if (e.getSource() == jcMem)
	    select_burg();

	if (e.getSource() == jcBand)
	    select_band();

        if (e.getSource() == jbRevc) {
            RevC = !RevC;
	    if (CS != null)
		PG.setColor();
        }

        if (e.getSource() == jbLog)
            set_log();

	enable_set();
	rewrite();
	PG.repaint();
    }
    public void set_gain() {
	Gain = 6-jcGain.getSelectedIndex();
	PW.setGain(Gain);
    }
    public void write_data() { // for Debug
	PrintWriter pw = null;
	String fn = "wave.dat";

        try {
	    CFL.calcFilter(Wave, FF, BB);
	    double val = Short.MAX_VALUE/Gain/CFL.getMaxData();
	    pw = new PrintWriter(new FileWriter(fn));
	    for (int i = 0; i < Wsize; i++)
		pw.println(CFL.getData(i)*val);

	    pw.close();
       } catch(IOException ex) {
	   Error = "File write error."; 
       }
    }
    public void play_wave() {
	if (isThread()) {
	    PW.Stop();
	    thread_stop();
	} else {
	    CFL.calcFilter(Wave, FF, BB);
	    PW = new Player(CFL.getData(), Gain, CFL.getMaxData());
	    PW.Start();
	    thread_start();
	}
	set_play_button();
    }
    public void set_play_button() {
	jbPlay.setText(isThread() ? "Stop" : "F.Play");
    }
    public void select_band() {
	int p = jcBand.getSelectedIndex();
	BB = (int)(Math.pow(2, p)/2);

	CALL = false;
    }
    public void set_filter() {
	Grap = FILT;
    }
    public void set_fft_button() {
	if (Grap == FFT_ || Grap == FFT3D) {
	    FftCnt++;
	    if (FftCnt > 1) {
		FftSW = !FftSW;
		FftCnt = 0;
	    }
	} else
	    FftCnt = 0;
    }
    public void enable_set() {
	boolean gs = (Grap == WAVE || Grap == FFT_ ||
		      Grap == FFT3D || Grap == FWT_ || 
		      Grap == FILT);
	jscStart.setEnabled(gs);

	boolean nc = (Grap != CWT_);
	jbPwide.setEnabled(nc);
	jbMwide.setEnabled(nc);

	boolean ps = (Grap == SPECT);
	jbRevc.setEnabled(ps);

	boolean ws = (Grap == CWT_ || Grap == FWT_);
	jcOctave.setEnabled(ws);

	boolean ds = (Grap == FFT3D || Grap == FFT2D || Grap == SPECT);
	jbLog.setEnabled(ds);
	jcDevid.setEnabled(ds);

	boolean ms = (Grap == MEM_);
	jcMem.setEnabled(ms);

	boolean fs = (Grap == FILT || Grap == FFT3D || Grap == FFT2D);
	jcBand.setEnabled(fs);

	boolean pl = (!isWave());
	jbPlay.setEnabled(pl);
    }
    //
    public void set_log() {
        LogSW = !LogSW;
        jbLog.setText(LogSW ? "Log " : "Sqrt");

	init_log();
        CALS = false;
    }
    public void init_log() {
	F3dV = LogSW ? LogV : SqrtV;
        DC = 1;
    }
    public void select_burg() {
	PG.set_burg_data(jcMem.getSelectedIndex());
    }
    public void select_devid() {
        int de = jcDevid.getSelectedIndex();
        Dev = (int)Math.pow(2, de); // 1/1~1/32

        CALS = false;
    }
    public void select_octave() {
	OctaveN = ((Integer)(jcOctave.getSelectedItem())).intValue();

	CALC = false;
	CALF = false;
    }
    public void select_sample() {
	NB = ((Integer)(jcSample.getSelectedItem())).intValue();
	set_nn();
    }
    public void set_nn() {
	NN = calc_nn();
	if (Wwide > 0)
	    sampl_check();

	Wtmp = new int[NN];
	N2 = NN/2;
	PG.reset_fft_m(0); // fft wide ꥢ
	if (CF != null) {
	    CF.initFFT(NN);

	    set_mmax();
	    CALM = false;
	}
	if (CFL != null) {
	    CFL.initFilter(NN);

	    CALL = false;
	}
	if (CS != null) {
	    CS.initSpectrogram(NN);

	    CALS = false;
	}
    }
    public int calc_nn() {
	return (int)Math.pow(2, NB);
    }
    public void sampl_check() {
        while (NN > Wwide) {
            NB--;
            NN = calc_nn();
            if (NB < 9) {
                Error = "NB error.";
                break;
            }
            for (int i = 0; i < Sample.length; i++) {
                if (NB == Sample[i])
                    jcSample.setSelectedIndex(i);
            }
        }
    }
    public void set_mmax() {
	Mmax = (int)Math.sqrt(NN); /* 2.0*Math.sqrt(NN) */
    }
    public void set_gain(boolean sw) {
	double s2 = Math.sqrt(2);

	switch (Grap) {
	case WAVE:
	case FILT:
	    WaveV = sw ? WaveV/s2 : WaveV*s2;
	    break;
	case FFT_:
	    FftV = sw ? FftV*s2 : FftV/s2;
	    break;
	case FFT2D:
	case FFT3D:
	    if (LogSW) {
		LogV = sw ? LogV*s2 : LogV/s2;
	    } else {
		SqrtV = sw ? SqrtV*s2 : SqrtV/s2;
	    }
	    F3dV = LogSW ? LogV : SqrtV;
	    break;
	case FWT_:
	    FwtV = sw ? FwtV*s2 : FwtV/2;
	    break;
	case CWT_:
	    Gainc = sw ? Gainc*s2 : Gainc/s2;
	    if (Gainc < 1)
		Gainc = 1;
	    break;
	case SPECT:
	    DC = sw ? DC*s2 : DC/s2;
            if (DC < 1)
                DC = 1;
            break;
	case MEM_:
	    MemV = sw ? MemV*s2 : MemV/s2;
	    break;
	default:
	}
    }
    public void set_wide(boolean sw) {
	int min = 2;
	int nn = (CF == null) ? min : NN;

	switch (Grap) {
	case WAVE:
	case FILT:
	case FWT_:
	    Wwide = sw ? Wwide*2 : Wwide/2;

	    if (Wwide > Wsize)
		Wwide = Wsize;
	    else if (Wwide < nn)
		Wwide = nn;

	    setScroll();
	    reset_all_sw();
	    break;
	case FFT_:
	case FFT2D:
	case FFT3D:
	case SPECT:
	case MEM_:
	    Wby = sw ? Wby/2 : Wby*2;
	 
	    if (Wby < min)
		Wby = min;
	    if (CF != null) {
		if (Wby > nn/2)
		    Wby = nn/2;
	    }
	    break;
	default:
	}
    }
    public void init_Scroll() {
        Wstart = 0;
        jscStart.setValues(Wstart, Wwide, 0, Wsize);
        jscStart.setBlockIncrement(Wwide/2);
    }
    public void reset_scroll() {
	switch (Grap) {
	case WAVE:
	case FILT:
	case FFT_:
	case FFT2D:
	case FFT3D:
	case FWT_:
	    jscStart.setValues(Wstart, Wwide, 0, Wsize);
	    jscStart.setBlockIncrement(Wwide/2);
	    break;
	default:
	}
    }
    public void setScroll() {
	switch (Grap) {
	case WAVE:
	case FILT:
	case FFT_:
	case FFT2D:
	case FFT3D:
	case FWT_:
	    Wstart = jscStart.getValue();

	    int max = Wsize-Wwide;
	    if (Wstart > max)
		Wstart = max;
	    jscStart.setValues(Wstart, Wwide, 0, Wsize);
	    jscStart.setBlockIncrement(Wwide/2);

	    reset_all_sw();
	    break;
	default:
	}
    }
    public void file_set() {
	WR = new WaveReader();
	Error = WR.getError();
	Fname = WR.getFile();
	if (Fname != null && Error == null) {
	    Splngf = WR.getSamplf();
	    Wave = WR.getData();
	    Wsize = Wwide = Wave.length;

	    set_nn();
	    init_Scroll();
	    reset_all_sw();
        } else {
	    Wave = null;
	    Wsize = Wwide = 0;
	}
	write_format(WR.getFormat());
    }
    public boolean isWave() {
	return (Wave == null);
    }
    public void reset_all_sw() {
	CALC = false;
	CALS = false;
	CALM = false;
	CALF = false;
	CALL = false;
    }
    //--
    public void rewrite() {
	switch (Grap) {
	case WAVE:
	    write_fft();
	    break;
	default:
	    write_filenm(Fname);
	    break;
	}
    }
    public void write_fft() {
	if (CF != null) {
	    double ds = 1.0/Splngf;
	    write_area("Time:"+dform(ds*Wsize)+"[sec] ("+dform(ds*NN)+")");
	}
    }
    public void write_format(AudioFormat f) {
	jtForm.setText((Error == null) ? ""+f : "Error:"+Error);
    }
    public void write_filenm(File fn) {
	jtFile.setText((Fname != null) ? fn.getName() : "Error:"+Error);
    }
    public void write_area(String st) {
	jtFile.setText(st);
    }
    public String dform(double val) {
	return df1.format((float)val);
    }
    public String dform0(double val) {
	return df0.format((float)val);
    }
    //------

    public class PaintGraph extends JPanel {
	final int Fnt = 12;
	final int Oct = 12;
	final int Color_max = 256; // for spect.
	final Color midGray = new Color(96, 96, 96);
	final Color Bar = new Color(64, 128, 255);
        final int[] R =     // for fwt
        {255,255,255,255,170, 90,  0,  0,  0,  0, 40, 90, 90,170,255};
        final int[] G = 
        {  0, 90,170,255,255,255,170,170,170, 90,  0,  0,  0,  0,  0};
        final int[] B = 
        {  0,  0,  0,  0,  0,  0, 90,170,255,255,255,255,220,200,170};

	int W, H, H2;
	int LeftS;
	int DownS;
	int ViewAnalX, ViewAnalY;
	int ColorN;         // for cwt/2-3dfft
	int Frame;
	int Mstart;         // mouse
	int Mend;           // "
	int Fstart;         // FFT
	int Fend;           // "
	int Xst = LeftS+40; // 3DFFT
	int Yst = 100;      // 3DFFT
	int Wide;           // NN/Wby
	int Spl;            // Splngf/Wby
	double DS;          // (W-LeftS)/(double)Spl
	double DW;          // (W-LeftS)/(double)Wide
	double Nsp;         // NN/(double)Splngf
	double MaxD, MinD;  // for spect
	double MaxValue;
	double[] burg;      // burg data
	Color[] Col;        // for spect.
	Color[] Grad;       // for cwt
	Color[] Octc;       // for fwt
	Font FNT = new Font("serif", Font.PLAIN, Fnt);
	FontMetrics fm = getFontMetrics(FNT);

	public PaintGraph() {
	    black_back();
	    initColor();
	    setColor();
	    set_color();
	    LeftS = get_lside();
        }
        public void black_back() {
            setBackground(Color.black);
        }
	public void dispColorbar(Graphics g) {
	    double dx = W/(double)ColorN;
	    int x = (int)dx;
	    int w = (x < 1) ? 1 : x; 
	    int y = H-Fnt*2;

	    for (int i = 0; i < ColorN; i++) {
		g.setColor(Grad[i]);
		g.fillRect((int)(dx*i), y, w, Fnt);
	    }   
	}
        public void initColor() {
            int f = 240, z = 255-f;
            ColorN = f*4;
            Grad = new Color[ColorN];
            for (int i = 0; i < f; i++) {
                Grad[i] = new Color(f, i, z);
                Grad[i+f] = new Color(f-i, f, z);
                Grad[i+f*2] = new Color(z, f, i);
                Grad[i+f*3] = new Color(z, f-i, f);
            }
	}
	public void setColor() {
	    Col = new Color[Color_max];
	    for (int i = 0; i < Color_max; i++)
		Col[RevC ? Color_max-1-i : i] = new Color(i,i,i);
        }
        public void set_color() {
            Octc = new Color[G.length];
            for (int i = 0; i < G.length; i++)
                Octc[i] = new Color(R[i], G[i], B[i]);
        }
	public void set_wides() {
	    Wide = NN/Wby;
	    Spl = Splngf/Wby;
	    DS = (W-LeftS)/(double)Spl;
	    DW = (W-LeftS)/(double)Wide;
	    Nsp = NN/(double)Splngf;
	}
	public int get_lside() {
	    String st = "00000";
	    return fm.stringWidth(st);
	}
        public void paintComponent(Graphics g) {
            Dimension d = getSize();
            W = d.width;
            H = d.height;
	    DownS = H-Fnt*3;
	    H2 = DownS/2;

	    super.paintComponent(g);
	    g.setFont(FNT);
	    set_wides();

	    switch (Grap) {
	    case WAVE:
		disp_wave(g);
		break;
	    case FFT_:
		disp_fft(g);
		break;
	    case FFT2D:
		disp_2dfft(g);
		break;
	    case FFT3D:
		disp_3dfft(g);
		break;
	    case FWT_:
		disp_fwt(g);
		break;
	    case CWT_:
		disp_wavelet(g);
		dispColorbar(g);
		break;
	    case SPECT:
		disp_spectrogram(g);
		break;
	    case MEM_:
		disp_mem(g);
		break;
	    case FILT:
		disp_filter(g);
		break;
	    default:
	    }
	    disp_play(g);
	}
	public void disp_lefts(Graphics g) {
	    g.setColor(Color.gray);
            g.drawLine(LeftS, 0, LeftS, DownS);
	}
	//
	public void disp_play(Graphics g) {
	    if (isWave() || PW == null)
                return;

	    if (PW.isRun()) {
		String pl = "Play";
		int xl = fm.stringWidth(pl);
		g.setColor(Color.gray);
		g.fill3DRect(LeftS+2, 1, xl+8, Fnt+6, true); 
		g.setColor(Color.orange);
		g.drawString(pl, LeftS+6, Fnt+2);
	    }
	}
	//
	public void disp_wave(Graphics g) {
	    g.setColor(Color.gray);
            g.drawLine(LeftS, H2, W, H2);
	    g.drawLine(LeftS, DownS, W, DownS);
	    disp_lefts(g);

	    if (isWave())
                return;

	    double dw = (W-LeftS)/(double)Wwide;
	    double wby = H2/WaveV;
	    int x, y, val, max = Integer.MIN_VALUE, min = Integer.MAX_VALUE;

            g.setColor(Color.green);
            for (int i = 0; i < Wwide; i++) {
		val = Wave[i+Wstart];
		x = (int)(LeftS+dw*i);
		y = (int)(H2-val*wby);
                g.drawLine(x, y, x, y); 
		max = Math.max(max, val);
		min = Math.min(min, val);
	    }

	    disp_time(g);
	    disp_wave_value(g, min, max, wby);
	    if (DragSW)
		disp_cursor(g, dw);
	}
	public void disp_cursor(Graphics g, double dw) {
	    if (Math.abs(get_LFD(dw)) < NN)
		g.setColor(Color.red);
	    else
		g.setColor(Bar);
	    g.drawLine(Firstx, 0, Firstx, H);
	    g.drawLine(Lastx, 0, Lastx, H);
	}
        public void disp_wave_value(Graphics g, int min, int max, double by) {
	    double up = max*by;
	    if (up > H2)
		up = H2-Fnt;
	    double dn = min*by;
	    if (-dn > H2)
		dn = -H2+Fnt;

            g.setColor(Color.lightGray);
            g.drawString(dform(100.0*max/Short.MAX_VALUE), 1, (int)(H2-up));
            g.drawString(dform(100.0*min/Short.MIN_VALUE), 1, (int)(H2-dn+Fnt));
	    g.drawString("[%]", 1, DownS+Fnt);
        }
	public void disp_time(Graphics g) {
	    double allsec = 100*Wwide/(double)Splngf; // 0.01 = 1
	    double ds = (W-LeftS)/allsec;
	    int x, bar = 8;

	    for (int i = 0; i <= allsec; i++) {
		if (i % 5000 == 0 && allsec > 5000) {
		    bar = 10;
		    disp_sec(g, i, ds);
		} else if (i % 1000 == 0 && allsec > 1000) { // 10 sec
		    bar = 8;
		    if (allsec < 9000)
			disp_sec(g, i, ds);
		} else if (i % 500 == 0 && allsec < 5000) {
		    bar = 7;
		    disp_sec(g, i, ds);
		} else if (i % 100 == 0 && allsec < 4000) { // 1
		    bar = 6;
		    if (allsec < 1000)
			disp_sec(g, i, ds);
		} else if (i % 50 == 0 && allsec < 1000) {
		    bar = 5;
		    if (allsec < 500)
			disp_sec(g, i, ds);
		} else if (i % 10 == 0 && allsec < 500) { // 0.1
		    bar = 4;
		    if (allsec < 100)
			disp_sec(g, i, ds);
		} else if (i % 5 == 0 && allsec < 90) {
		    bar = 3;
		    if (allsec < 10)
			disp_sec(g, i, ds);
		} else if (allsec < 50) {            //  0.01
		    bar = 2;
		    if (allsec < 10)
			disp_sec(g, i, ds);
		} else
		    continue;

		x = (int)(ds*i+LeftS);
		g.setColor(Color.lightGray);
		g.drawLine(x, DownS, x, DownS+bar);
	    }

	    disp_sec(g);
	}
	public void disp_sec(Graphics g, int sec, double ds) {
	    g.setColor(Color.yellow);
	    g.drawString(dform(sec/100.0), (int)(ds*sec+LeftS), H-2);
	}
	public void disp_sec(Graphics g) {
	    String sec = "[sec] ";
	    int xl = fm.stringWidth(sec);

	    g.setColor(Color.yellow);
	    g.drawString(sec, LeftS-xl, H);
	}
	//
	public void disp_filter(Graphics g) {
	    g.setColor(Color.gray);
            g.drawLine(LeftS, H2, W, H2);
	    g.drawLine(LeftS, DownS, W, DownS);
	    disp_lefts(g);

	    if (isWave())
                return;
	    if (!CALL) {
		CFL.calcFilter(Wave, FF, BB);
		CALL = true;
	    }
	    double dw = (W-LeftS)/(double)Wwide;
	    double val, wby = H2/WaveV;
	    int x, y, max = Integer.MIN_VALUE, min = Integer.MAX_VALUE;

            g.setColor(Color.cyan);
            for (int i = 0; i < Wwide; i++) {
		val = CFL.getData(Wstart+i);
		x = (int)(LeftS+dw*i);
		y = (int)(H2-val*wby);
                g.drawLine(x, y, x, y);
		max = Math.max(max, (int)val);
		min = Math.min(min, (int)val);
	    }

	    disp_time(g);
	    disp_wave_value(g, min, max, wby);
	    if (DragSW)
		disp_cursor(g, dw);
	}
	//---
	public void calc_fft() {
	    int[] wv = new int[NN];
	    System.arraycopy(Wave, Wstart, wv, 0, NN); 
	    CF.fft(wv);
	}
	public void disp_fft(Graphics g) {
	    disp_lefts(g);
	    if (isWave() | CF == null)
                return;

	    calc_fft();
	    double dt0, dt1 = 0;
	    int x0, x1 = 0;
	    int ww = (DW < 1) ? 1 : (int)DW;
	    for (int i = 0; i < Wide; i++) {
		dt0 = CF.getData(i)*FftV;
		x0 = (int)(LeftS+DW*i);
		g.setColor((i > Fstart && i < Fend) ?
			   Color.white : Color.green);
		if (i == 0) {
		    x1 = x0;
		    dt1 = dt0;
                }
		if (FftSW)
		    g.drawLine(x0, (int)(DownS-dt0), x1, (int)(DownS-dt1));
		else
		    g.drawRect(x0, (int)(DownS-dt0), ww, (int)dt0);
		x1 = x0;
		dt1 = dt0;
	    }

	    disp_fft_memori(g, LeftS);
	    disp_hz(g, H);
	}
	public void disp_fft_memori(Graphics g, int xsf) {
	    int spi, l;
	    double dw = (W-xsf)/(double)Spl;

	    for (int i = 0; i < Spl; i += 10) {
		if (i % 10000 == 0) {
                    l = 10;
                    if (Spl >= 10000)
                        disp_k(g, i, (int)(xsf+dw*i), H);
                } else if (i % 1000 == 0) {
                    l = 8;
                    if (Spl < 12000)
                        disp_k(g, i, (int)(xsf+dw*i), H);
                } else if (i % 500 == 0)
                    l = 6;
                else if (i % 100 == 0)
                    l = 4;
                else if (Spl < 2000)
		    l = 2;
		else
                    continue;
                spi = (int)(xsf+dw*i);
                g.setColor(Color.gray);
                g.drawLine(spi, DownS, spi, DownS+l);
            }
	}
	public void disp_k(Graphics g, int k, int x, int y) {
            g.setColor(Color.pink);
            g.drawString(k/1000+"k", x, y);
        }
	public void set_m_dragged(int st) {
	    if (st < LeftS)
		st = LeftS;
	    if (st > W)
		st = W;
	    if (st > Mstart)
		Mend = st;
	    if (st < Mend)
		Mstart = st;

	    set_f_dragged();
	}
	public void set_f_dragged() {
	    Fstart = (int)((Mstart-LeftS)/DW);
	    Fend = (int)((Mend-LeftS)/DW);
	}
	public void reset_fft_m(int x) {
	    Mstart = Mend = x;

	    set_f_dragged();
	}
	//---

	public void disp_2dfft(Graphics g) {
	    if (isWave() || CF == null)
		return;
	    if (!CALS)
		calc_spect();

	    int x0, y0, x1 = 0, y1 = 0, jj = 0;
	    int bb = BB*2;
	    if (bb > Wide)
		bb = Wide;
	    else if (bb < 1)
		bb = 1;
	    double yy;
	    double dy = H2/(double)bb;
	    double dx = (W-LeftS)/(double)(Frame-1);
	    for (int j = FF-BB; j <= FF+BB; j++) { // h
		if (j < 0 || j >= Wide)
		    continue;
		for (int i = 0; i < Frame; i++) { // w
		    x0 = (int)(LeftS+dx*i);
		    yy = H2+dy*jj;
		    y0 = (int)(yy-CS.getData(j+i*N2)*F3dV);
		    set_fft_color(g, i, j);
		    if (i == 0) {
                        x1 = x0;
                        y1 = y0;
                    }
		    g.drawLine(x0, y0, x1, y1);
		    g.drawLine(x0, (int)yy, x0, y0);
		    x1 = x0;
                    y1 = y0;
                }
		jj++;
            }

	    disp_2d_memori(g, bb, dy);
	    disp_time(g);
	    disp_hz(g, H-Fnt*2);
	}
	public void disp_2d_memori(Graphics g, int b2, double dh) {
	    int y0, l = 5;

	    g.setColor(Color.gray);
	    for (int i = 0; i <= b2; i++) {
		if (i % 1000 == 0)
		    l = 10;
		else if (i % 500 == 0)
		    l = 8;
		else if (i % 100 == 0)
		    l = 6;
		else if (i % 10 == 0) {
		    l = 4;
		    if (b2 > 1000)
			continue;
		} else if (b2 < 100) {
		    l = 2;
		} else
		    continue;

		y0 = (int)(H2+dh*i);
		g.drawLine(LeftS-1, y0, LeftS-l, y0);
	    }

	    int ct = (int)(H2+dh*BB);
	    if (FF < BB)
		ct = (int)(ct-dh*(BB-FF));
	    else if (FF+BB > Wide)
		ct = (int)(ct+dh*(FF+BB-Wide));
	    g.setColor(Color.lightGray);
	    g.drawLine(0, ct, LeftS-5, ct);

	    g.setColor(Color.pink);
	    g.drawString(dform0(FF/Nsp), 1, ct);
	}
	//---

	public void cursolSet(int x) {
	    FF = (int)((x-LeftS)/DW);
	    if (FF < 0)
		FF = 0;
	    else if (FF >= Wide)
		FF = Wide;
	}
	public void disp_3dfft(Graphics g) {
	    if (isWave() || CF == null)
		return;
	    if (!CALS)
		calc_spect();

	    int x0, y0, x1 = 0, y1 = 0, ni;
	    double dx = (W-Xst)/(double)Wide;
            double xsf = (Xst-LeftS)/(double)Frame, xx, yy;
            double ysf = (DownS-Yst)/(double)Frame;
	    boolean wsw = (Mstart == Mend); //
	    int ww = (dx < 1) ? 1 : (int)dx;
	    for (int i = 0; i < Frame; i++) {
		yy = Yst+ysf*i;
                xx = LeftS+xsf*i;
		ni = i*N2;
		for (int j = 0; j < Wide; j++) {
		    x0 = (int)(xx+dx*j);
		    if (!wsw) {
			y0 = (j > Fstart && j < Fend) ?
			    (int)(yy-CS.getData(j+ni)*F3dV) : (int)yy;
		    } else {
			y0 = (int)(yy-CS.getData(j+ni)*F3dV);
		    }
		    set_fft_color(g, i, j);
		    
		    if (j == 0) {
                        x1 = x0;
                        y1 = y0;
                    }
		    if (FftSW)
			g.drawLine(x0, y0, x1, y1);
		    else
			g.drawRect(x0, y0, ww, (int)(yy-y0));
                    x1 = x0;
                    y1 = y0;
                }
            }

	    g.setColor(Color.gray);
	    g.drawLine(LeftS, Yst, Xst, DownS);
	    disp_fft_memori(g, Xst);
	    disp_3d_time(g);
	}
	public void set_fft_color(Graphics g, int i, int j) {
	    if (FF-BB > j || FF+BB < j)
		g.setColor(midGray);
	    else {
		int stp = (int)(i*ColorN/(double)Frame);
		g.setColor(Grad[stp]);
	    }
	}
	public void set_3d_xy(int mx, int my) {
	    Xst = mx;
	    Yst = my;
	}
	public void disp_3d_time(Graphics g) {
	    double tim, allsec = Wwide/(double)Splngf;

	    if (allsec < 0.1)
		tim = 0.01;
	    else if (allsec < 0.3)
		tim = 0.05;
	    else if (allsec < 1)
		tim = 0.1;
	    else if (allsec < 3)
		tim = 0.5;
	    else if (allsec < 10)
		tim = 1;
	    else if (allsec < 30)
		tim = 5;
	    else if (allsec < 60)
		tim = 10;
	    else
		tim = 60;

	    double maxsec = allsec/tim;
	    double asec = (DownS-Yst)/maxsec;
	    double dx = (Xst-LeftS)/maxsec;
	    if (maxsec > 0) {
		int x, y, l = 5;
		for (int i = 0; i <= maxsec; i++) {
		    y = (int)(asec*i+Yst);
		    x = (int)(LeftS+dx*i);
		    g.setColor(Color.gray);
		    g.drawLine(x, y, x-l, y); 

		    g.setColor(Color.yellow);
		    g.drawString(dform(i*tim), x-30, y+Fnt);
		}
	    }
	}
	//---
	public void calc_spect() {
	    CS = new CalcSpectrogram(Wave, NN);
	    CS.calcSpectrogram(Dev, Wstart, Wwide, LogSW);
	    PG.Frame = CS.getFrame();
	    PG.MaxD = CS.getMaxD();
	    PG.MinD = CS.getMinD();

	    CALS = true;
	}
	public void disp_spectrogram(Graphics g) {
	    if (isWave())
                return;
	    if (!CALS)
		calc_spect();
	    int x0, ni;
	    double df = (W-LeftS)/(double)Frame;
	    double dh = DownS/(double)Wide;
            double dhs = DownS-dh;
            int[] y0 = new int[Wide];

            for (int k = 0; k < Wide; k++)
                y0[k] = (int)(dhs-dh*k);
	    int bnd = (df < 1) ? 1 : (int)(df+1);
	    int hgt = (dh < 1) ? 1 : (int)(dh+1);
            for (int i = 0; i < Frame; i++) {
                x0 = (int)(LeftS+df*i);
		ni = i*N2;
                for (int k = 0; k < Wide; k++) {
		    set_color(g, CS.getData(k+ni));
		    g.fillRect(x0, y0[k], bnd, hgt);
		}
	    }

	    disp_vmemori(g);
	    disp_time(g);
	    disp_hz(g, DownS+Fnt);
	}
	public void set_color(Graphics g, double data) {
	    int limit = 255; // not 256
	    double dc = limit/((MaxD-MinD)/DC);
	    int co = (int)(dc*(data-MinD));

	    if (co > limit)
		co = limit;
	    else if (co < 0)
		co = 0;
	    g.setColor(Col[co]);
	}
	public void disp_vmemori(Graphics g) {
	    int spi, l;
	    double dm = DownS/(double)Spl;

	    for (int i = 0; i < Spl; i += 100) {
		if (i % 10000 == 0) {
		    l = 10;
		    if (Spl >= 10000)
			disp_k(g, i, 1, (int)(DownS-dm*i));
		} else if (i % 1000 == 0) {
		    l = 7;
		    if (Spl < 12000)
			disp_k(g, i, 1, (int)(DownS-dm*i));
		} else if (i % 500 == 0)
		    l = 4;
		else if (i % 100 == 0)
		    l = 2;
		else
		    continue;
		spi = (int)(DownS-dm*i);
		g.setColor(Color.gray);
		g.drawLine(LeftS, spi, LeftS-l, spi);
	    }
	}
	//---
	public void calc_fwt() {
	    int[] temp = new int[Wwide];
	    System.arraycopy(Wave, Wstart, temp, 0, Wwide);
	    FW.calcWavelet(temp, Wwide, OctaveN);

	    CALF = true;
	}
	public void disp_fwt(Graphics g) {
	    disp_lefts(g);
	    if (isWave())
		return;
	    if (!CALF)
		calc_fwt();

	    int x1, y1, sj = Splngf/2, lj = Wwide;
	    double dx, h1, hs = DownS/(double)OctaveN;

	    for (int j = 1; j <= OctaveN; j++) {
                lj /= 2;
		dx = (W-LeftS)/(double)lj;
                h1 = hs*j;

                g.setColor(Color.gray); // line
                g.drawLine(0, (int)h1, W, (int)h1);

		sj /= 2;
		g.setColor(Color.pink); // samplf
		g.drawString(""+sj, 0, (int)(h1+Fnt));

		g.setColor(Octc[j]);
                for(int i = 0; i < lj; i++) {
		    x1 = (int)(LeftS+i*dx);
                    y1 = (int)(h1-FW.getData(i+lj)*FwtV);
                    g.drawLine(x1, (int)h1, x1, y1);
                }
	    }

	    disp_time(g);
	    disp_hz(g, Fnt);
	}
	//---
	public void calc_cwt() {
	    CW = new CalcWavelet(Wave, Wsize);
	    CW.calcWavelet(Wstart, OctaveN, Wwide);
	    PG.ViewAnalX = CW.getViewX();
	    PG.ViewAnalY = CW.getViewY();
	    PG.MaxValue = CW.getMaxValue();

	    CALC = true;
	}
	public void disp_wavelet(Graphics g) {
	    if (isWave())
		return;
	    if (!CALC)
		calc_cwt();

	    double dx = (double)(W-LeftS)/ViewAnalX;
	    double dy = (double)DownS/ViewAnalY;
	    double dc = ColorN/MaxValue;
	    int[] x0 = new int[ViewAnalX];
	    for(int j = 0; j < ViewAnalX; j++)
		x0[j] = (int)(dx*j+LeftS);

	    int l = 0, y0;
	    int max = ColorN-1;
	    for(int i = 0; i < ViewAnalY; i++) {
		y0 = (int)(dy*i);
		for(int j = 0; j < ViewAnalX; j++) {
		    g.setColor(set_color(dc, CW.getData(l++), max));
		    g.drawLine(x0[j], y0, x0[j], y0);
		}
	    }

	    disp_time(g);
	    disp_y_line(g);
	}
        public Color set_color(double dc, double num, int max) {
	    int val = (int)(dc*num*Gainc);
            if (val > max)
                val = max;

            return Grad[max-val]; // 顼ȿž
        }
	public void disp_y_line(Graphics g) {
	    double dh = (double)DownS/OctaveN;
	    int dy, spl = Splngf;

	    for (int i = 0; i <= OctaveN; i++) {
		dy = (int)(dh*i);
		g.setColor(Color.gray);
		g.drawLine(0, dy, LeftS, dy);

		spl /= 2;
		g.setColor(Color.pink);
		g.drawString(""+spl, 0, dy+Fnt);
	    }
	}
	public void disp_hz(Graphics g, int h) {
	    g.setColor(Color.pink);
	    g.drawString("[Hz]", 2, h);
	}
	//---
	public void calc_mem() {
	    MB = new MemBurg();
	    MB.setNN(NN, Splngf, Mmax);
	    MB.calcMem(Wstart, Wave);
	    PG.set_burg_data(jcMem.getSelectedIndex());

	    CALM = true;
	}
	public void disp_mem(Graphics g) {
	    disp_lefts(g);
	    if (isWave())
                return;
	    if (!CALM)
		calc_mem();

	    disp_burg_data(g);
	    double fw = (W-LeftS)/(Nsp*Spl);
	    double x0, y0, x1 = 0, y1 = 0;

	    g.setColor(Color.green);
	    for (int i = 0; i < Wide; i++) {
		x0 = LeftS+fw*i;
		y0 = DownS-MB.getData(i+1)*MemV;
		if (i == 0) {
		    x1 = x0;
		    y1 = y0;
		}
		g.drawLine((int)x0, (int)y0, (int)x1, (int)y1);
		x1 = x0;
		y1 = y0;
	    }

	    disp_fft_memori(g, LeftS);
	    disp_hz(g, H);
	}
	public void set_burg_data(int sw) {
	    switch(sw) {
	    case 0: // G
		burg = MB.getG();
		break;
	    case 1: // C
		burg = MB.getC();
		break;
	    case 2: // FPE
		burg = MB.getFPE();
		break;
	    case 3: // AIC
		burg = MB.getAIC();
		break;
	    default:
		System.out.println("No data.");
	    }
	}
	public void disp_burg_data(Graphics g) {
	    int len = burg.length-1;
	    double dd = (W-LeftS)/(double)len;
	    double[] dhc = disp_minmax(g, len); // dh, dc

	    g.setColor(Color.darkGray);
	    g.drawLine(LeftS, H2, W, H2);

	    int x0, y0, x1 = 0, y1 = 0;
	    g.setColor(Color.gray);
	    for (int i = 0; i <= len; i++) {
		x0 = (int)(LeftS+dd*i);
		if (i == 0) {
		    x1 = x0;
		    continue;
		}
		y0 = (int)(H2-(burg[i]-dhc[1])*dhc[0]); // dc, dh
		if (i == 1) {
		    y1 = y0;
		}
		g.drawLine(x0, y0, x1, y0);
		g.drawLine(x1, y0, x1, y1);
		x1 = x0;
		y1 = y0;
	    }
	}
	public double[] disp_minmax(Graphics g, int len) {
	    double max = Double.MIN_VALUE, min = Double.MAX_VALUE;
	    for (int i = 1; i <= len; i++) {
		max = Math.max(max, burg[i]);
		min = Math.min(min, burg[i]);
	    }
	    double[] tmp = new double[2];
	    tmp[0] = H2/(max-min); // dh
	    tmp[1] = (max+min)/2;  // dc

	    String mx = "Mmax:"+Mmax+" max:"+(float)max+" ";
	    String mn = "min:"+(float)min+" ";
	    int xl = fm.stringWidth(mx);
	    int nl = fm.stringWidth(mn);
	    g.setColor(Color.lightGray);
	    g.drawString(mx, W-xl, Fnt);
	    g.drawString(mn, W-nl, DownS-Fnt);

	    return tmp;
	}
    }
}